@@ -3,10 +3,10 @@
  *
  * Type definitions and function declarations relating to date and time.
  *
- * $Id: time.h,v bfab21e13fbe 2016/08/24 23:01:17 keithmarshall $
+ * $Id: time.h,v bbbe71f5f21f 2018/09/02 19:28:38 keith $
  *
  * Written by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
- * Copyright (C) 1997-2007, 2011, 2015, 2016, MinGW.org Project.
+ * Copyright (C) 1997-2007, 2011, 2015-2018, MinGW.org Project.
  *
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -423,6 +423,40 @@ int nanosleep( const struct timespec *period, struct timespec *residual )
     ? (unsigned)(-1) : (unsigned)(period->tv_nsec));
 }
 #endif	/* !__NO_INLINE__ */
+
+#if _POSIX_C_SOURCE >= 199309L
+/* POSIX.1b-1993 introduced the optional POSIX clocks API; it
+ * was subsequently moved to "base", as of POSIX.1-2008, to the
+ * extent required to support the CLOCK_REALTIME feature, with
+ * the remainder of its features remaining optional.  We choose
+ * to provide a subset, supporting CLOCK_MONOTONIC in addition
+ * to the aforementioned CLOCK_REALTIME feature.
+ *
+ * We define the POSIX clockid_t type as a pointer to an opaque
+ * structure; user code should never need to know details of the
+ * internal layout of this structure.
+ */
+typedef struct __clockid__ *clockid_t;
+
+/* The standard clockid_t entities which we choose to support.
+ */
+extern clockid_t const CLOCK_REALTIME;
+extern clockid_t const CLOCK_MONOTONIC;
+
+/* Ensure that these clock implementations are detectable via
+ * preprocessor #ifdef names.
+ */
+#define CLOCK_REALTIME CLOCK_REALTIME
+#define CLOCK_MONOTONIC CLOCK_MONOTONIC
+
+/* Prototypes for the standard POSIX functions which provide the
+ * API to these standard clockid_t entities.
+ */
+int clock_getres (clockid_t, struct timespec *);
+int clock_gettime (clockid_t, struct timespec *);
+int clock_settime (clockid_t, const struct timespec *);
+
+#endif	/* _POSIX_C_SOURCE >= 199309L */
 #endif	/* _POSIX_C_SOURCE */
 
 _END_C_DECLS
